#include <stdio.h>
#include <gtk/gtk.h>
#include <gtkdatabox.h>
#include <math.h>
/* Some versions of math.h have a PI problem... */
#ifndef PI
#define PI 3.14159265358979323846
#endif

static void menu_color_change_cb		(gpointer callback_data, guint callback_action, GtkWidget *widget);

static GtkItemFactoryEntry
menu_main_entries[]=
{
  {"/_Color", 				NULL, NULL, 		0, 	"<Branch>"},
  {"/Color/Change Color #0",		NULL, menu_color_change_cb, 	0, 	NULL},
  {"/Color/Change Color #1",		NULL, menu_color_change_cb,	1, 	NULL},
  {"/Color/Change Color #2",		NULL, menu_color_change_cb,	2, 	NULL},
  {"/Color/Change Color #3",		NULL, menu_color_change_cb,	3, 	NULL},
  {"/Color/Change Color #4",		NULL, menu_color_change_cb,	4, 	NULL},
  {"/_Style", 				NULL, NULL, 		0, 	"<Branch>"},
  {"/Style/Graph #0", 			NULL, NULL, 		0, 	"<Branch>"},
  {"/Style/Graph #0/Dots", 		NULL, NULL, 		0, 	NULL},
  {"/Style/Graph #0/Lines", 		NULL, NULL, 		0, 	NULL},
  {"/_Options", 			NULL, NULL, 		0, 	"<Branch>"},
  {"/Options/Show coordinates",		NULL, NULL, 		0, 	"<RadioItem>"},
  {"/Options/Hide coordinates",		NULL, NULL, 		0, 	"/Options/Show coordinates"},
  {"/_Help", 				NULL, NULL, 				0, 	"<LastBranch>"},
  {"/Help/Do not hesitate to ask\nrbock@eudoxos.de", 	NULL, NULL, 		0, 	NULL},
};

static gint
menu_main_num = sizeof (menu_main_entries) / sizeof (menu_main_entries[0]);

typedef struct {
  GtkWidget *selector;
  GtkWidget *box;
  gint index;
} col_sel;

static void 
get_color_cb(GtkWidget *widget, col_sel *sel)
{
  gdouble color[8];
  
  g_return_if_fail(GTK_IS_COLOR_SELECTION_DIALOG(sel->selector));
  g_return_if_fail(GTK_IS_DATABOX(sel->box));

  gtk_color_selection_get_color(GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(sel->selector)->colorsel), color);
  gtk_databox_set_color(GTK_DATABOX(sel->box), sel->index, (gushort)(color[0]*65535), (gushort)(color[1]*65535), (gushort)(color[2]*65535));
  gtk_databox_redraw(GTK_DATABOX(sel->box));
}

static void
menu_color_change_cb(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  GtkWidget *selector;
  gchar title[20];
  col_sel *sel=(col_sel *) g_malloc(sizeof(col_sel));
  gushort red, green, blue;
  gdouble color[8];
  
  sprintf(title, "Choose color #%d", callback_action);
  selector=gtk_color_selection_dialog_new(title);
  gtk_widget_destroy(GTK_COLOR_SELECTION_DIALOG(selector)->help_button);
  sel->selector=selector;
  sel->box=GTK_WIDGET(callback_data);
  sel->index=callback_action;
  
  gtk_databox_get_color(GTK_DATABOX(sel->box), sel->index, &red, &green, &blue);
  color[0]=red/65535.;
  color[1]=green/65535.;
  color[2]=blue/65535.;
  color[3]=1;		/* Opacity? Dunno how to use it. */
  gtk_color_selection_set_color(GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(sel->selector)->colorsel), color);
  
  gtk_signal_connect_object(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(selector)->cancel_button), "clicked",
		 GTK_SIGNAL_FUNC(gtk_widget_destroy),
		 GTK_OBJECT(selector));
  gtk_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(selector)->ok_button), "clicked",
		 GTK_SIGNAL_FUNC(get_color_cb),
		 (gpointer) sel);
  gtk_widget_show(selector);

  return;
}

static void
do_exit (GtkWidget *widget, GtkWidget *window)
{
  gtk_widget_destroy (window);
  gtk_main_quit ();
}

static void
create_standard_window(gchar *title, GtkWidget **window, GtkWidget **box1, GtkWidget **box2, GtkWidget **close_button)
{
  GtkWidget *separator;
  
  if (!title || !window || !box1 || !box2|| !close_button) return;
  
  if (!*window)
  {
    *window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

    gtk_signal_connect (GTK_OBJECT (*window), "destroy",
			  GTK_SIGNAL_FUNC(gtk_widget_destroyed),
			  &*window);

    gtk_window_set_title (GTK_WINDOW (*window), title);
    gtk_container_border_width (GTK_CONTAINER (*window), 0);

    *box1 = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (*window), *box1);
    separator = gtk_hseparator_new ();

    *box2 = gtk_vbox_new (FALSE, 10);
    gtk_container_border_width (GTK_CONTAINER (*box2), 10);
    gtk_box_pack_end (GTK_BOX (*box1), *box2, FALSE, TRUE, 0);
    gtk_box_pack_end (GTK_BOX (*box1), separator, FALSE, TRUE, 0);

    *close_button = gtk_button_new_with_label ("close");
    gtk_signal_connect_object (GTK_OBJECT (*close_button), "clicked",
			 GTK_SIGNAL_FUNC(gtk_widget_destroy),
				 GTK_OBJECT (*window));
    gtk_box_pack_start (GTK_BOX (*box2), *close_button, TRUE, TRUE, 0);
    GTK_WIDGET_SET_FLAGS (*close_button, GTK_CAN_DEFAULT);
    gtk_widget_grab_default (*close_button);
  }
    
  return;
}

static void
show_standard_window(GtkWidget *window)
{
  if (!window) return;
  
  if (!GTK_WIDGET_VISIBLE (window))
    gtk_widget_show_all (window);
  else
    gtk_widget_destroy (window);

  return;
}

static void
create_basics(void)
{
  static GtkWidget *window = NULL;
  static GtkWidget *box1=NULL;
  static GtkWidget *box2=NULL;
  static GtkWidget *close_button=NULL;
  GtkWidget *label;
  GtkWidget *separator;
  GtkWidget *box;
  gint points=2000;
  gint i;
  
  create_standard_window("Basics", &window, &box1, &box2, &close_button);
  
  box = gtk_databox_new ();
  gtk_widget_set_usize (window, 300, 300);  
  
  label=gtk_label_new("Draw a selection with the left button pressed,\nThan click into the selection.\nUse the right mouse button to zoom out.\nShift+mouse button zooms to default.");
  gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 0);
  separator=gtk_hseparator_new();
  gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, FALSE, 0);
  
  GTK_DATABOX(box)->data.dimX=points;
  GTK_DATABOX(box)->data.dimY=1;

  GTK_DATABOX(box)->data.X=(gfloat **) g_malloc(GTK_DATABOX(box)->data.dimY*sizeof(gfloat *));
  GTK_DATABOX(box)->data.X[0]=(gfloat *) g_malloc(points*sizeof(gfloat));
  GTK_DATABOX(box)->data.Z=(gfloat **) g_malloc(GTK_DATABOX(box)->data.dimY*sizeof(gfloat *));
  GTK_DATABOX(box)->data.Z[0]=(gfloat *) g_malloc(points*sizeof(gfloat));
  for (i = 0; i < points; i++ ) {
     GTK_DATABOX(box)->data.X[0][i]=i;
     GTK_DATABOX(box)->data.Z[0][i]=100.*sin(i*2*PI/points);
  }
  
  gtk_databox_new_data(GTK_DATABOX(box), GTK_DATABOX_POINTS_2D);

  gtk_box_pack_start (GTK_BOX (box1), box, TRUE, TRUE, 0);
  
  show_standard_window(window);
}

static void
create_coordinates(void)
{
  static GtkWidget *window = NULL;
  static GtkWidget *box1=NULL;
  static GtkWidget *box2=NULL;
  static GtkWidget *close_button=NULL;
  GtkWidget *label;
  
  create_standard_window("Coordinates", &window, &box1, &box2, &close_button);
  
  label=gtk_label_new("This has to be coded yet :-)");
  gtk_box_pack_start (GTK_BOX (box1), label, TRUE, TRUE, 0);
  
  show_standard_window(window);
}

static void
create_drawing_styles(void)
{
  static GtkWidget *window = NULL;
  static GtkWidget *box1=NULL;
  static GtkWidget *box2=NULL;
  static GtkWidget *close_button=NULL;
  GtkWidget *separator;
  GtkWidget *box;
  GtkWidget *label;
  GtkWidget *menu_bar;
  GtkItemFactory *item_factory;
  gint i;
  gint points=2000;
  gchar title[]="<Main>";
  
  create_standard_window("Drawing Styles", &window, &box1, &box2, &close_button);
  gtk_widget_set_usize (window, 300, 300);  
  box = gtk_databox_new();
  
  item_factory=gtk_item_factory_new(GTK_TYPE_MENU_BAR, title, NULL);
  gtk_item_factory_create_items(item_factory, menu_main_num, menu_main_entries, box);
  menu_bar=gtk_item_factory_get_widget(item_factory, title);
  gtk_box_pack_start(GTK_BOX (box1), menu_bar, FALSE, TRUE, 0);

  
  label=gtk_label_new("You may change the colors of the shown datasets,\nthe drawing style and behaviour of\ncoordinate axes and/or scrollbars.");
  gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 0);
  separator=gtk_hseparator_new();
  gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, FALSE, 0);

  GTK_DATABOX(box)->data.dimX=points;
  GTK_DATABOX(box)->data.dimY=5;

  GTK_DATABOX(box)->data.X=(gfloat **) g_malloc(GTK_DATABOX(box)->data.dimX*sizeof(gfloat *));
  GTK_DATABOX(box)->data.X[0]=(gfloat *) g_malloc(points*sizeof(gfloat));
  for (i=1; i<GTK_DATABOX(box)->data.dimX; i++) 
    GTK_DATABOX(box)->data.X[i]=GTK_DATABOX(box)->data.X[0];

  GTK_DATABOX(box)->data.Z=(gfloat **) g_malloc(GTK_DATABOX(box)->data.dimX*sizeof(gfloat *));
  for (i=0; i<GTK_DATABOX(box)->data.dimX; i++) 
    GTK_DATABOX(box)->data.Z[i]=(gfloat *) g_malloc(points*sizeof(gfloat));

  for (i = 0; i < points; i++ ) {
     GTK_DATABOX(box)->data.X[0][i]=i;
     GTK_DATABOX(box)->data.Z[0][i]=100.*sin(i*2*PI/points);
     GTK_DATABOX(box)->data.Z[1][i]=100.*sin(i*4*PI/points);
     GTK_DATABOX(box)->data.Z[2][i]=100.*sin(i*6*PI/points);
     GTK_DATABOX(box)->data.Z[3][i]=100.*sin(i*8*PI/points);
     GTK_DATABOX(box)->data.Z[4][i]=100.*sin(i*10*PI/points);
  }
  
  gtk_databox_new_data(GTK_DATABOX(box), GTK_DATABOX_POINTS_2D);

  gtk_box_pack_start (GTK_BOX (box1), box, TRUE, TRUE, 0);
  
  show_standard_window(window);
}

static void
create_oszilloscope(void)
{
  static GtkWidget *window = NULL;
  static GtkWidget *box1=NULL;
  static GtkWidget *box2=NULL;
  static GtkWidget *close_button=NULL;
  GtkWidget *label;
  
  create_standard_window("Oscilloscope", &window, &box1, &box2, &close_button);
  
  label=gtk_label_new("This has to be coded yet :-)");
  gtk_box_pack_start (GTK_BOX (box1), label, TRUE, TRUE, 0);
  
  show_standard_window(window);
}

static void
create_main_window (void)
{
  struct {
    char *label;
    void (*func) ();
  } buttons[] =
    {
      { "databox basics", create_basics },
      { "databox coordinates", create_coordinates },
      { "databox drawing styles", create_drawing_styles },
      { "oszilloscope", create_oszilloscope },
    };
  int nbuttons = sizeof (buttons) / sizeof (buttons[0]);
  GtkWidget *window;
  GtkWidget *box1;
  GtkWidget *box2;
  GtkWidget *scrolled_window;
  GtkWidget *button;
  GtkWidget *separator;
  int i;

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_policy (GTK_WINDOW (window), FALSE, FALSE, FALSE);
  gtk_widget_set_name (window, "Gtk Databox");
  gtk_widget_set_usize (window, 200, 200);
  gtk_widget_set_uposition (window, 20, 20);

  gtk_signal_connect (GTK_OBJECT (window), "destroy",
		      GTK_SIGNAL_FUNC(gtk_main_quit),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (window), "delete-event",
		      GTK_SIGNAL_FUNC (gtk_false),
		      NULL);

  box1 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window), box1);

  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_container_border_width (GTK_CONTAINER (scrolled_window), 10);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
     		                  GTK_POLICY_AUTOMATIC, 
                                  GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start (GTK_BOX (box1), scrolled_window, TRUE, TRUE, 0);

  box2 = gtk_vbox_new (FALSE, 0);
  gtk_container_border_width (GTK_CONTAINER (box2), 10);
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled_window), box2);
  gtk_container_set_focus_vadjustment (GTK_CONTAINER (box2),
				       gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scrolled_window)));
  gtk_widget_show (box2);

  for (i = 0; i < nbuttons; i++)
    {
      button = gtk_button_new_with_label (buttons[i].label);
      if (buttons[i].func)
        gtk_signal_connect (GTK_OBJECT (button), 
			    "clicked", 
			    GTK_SIGNAL_FUNC(buttons[i].func),
			    NULL);
      else
        gtk_widget_set_sensitive (button, FALSE);
      gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);
    }

  separator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);

  box2 = gtk_vbox_new (FALSE, 10);
  gtk_container_border_width (GTK_CONTAINER (box2), 10);
  gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, TRUE, 0);

  button = gtk_button_new_with_label ("close");
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (do_exit),
		      window);
  gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (button);

  gtk_widget_show_all (window);
}

gint 
main(gint argc, char *argv[])
{
  gtk_init(&argc, &argv);
  
  create_main_window();
  
  gtk_main();

  return 0;
}